############################################################################
##
## Copyright (C) 2006-2007 University of Utah. All rights reserved.
##
## This file is part of VisTrails.
##
## This file may be used under the terms of the GNU General Public
## License version 2.0 as published by the Free Software Foundation
## and appearing in the file LICENSE.GPL included in the packaging of
## this file.  Please review the following to ensure GNU General Public
## Licensing requirements will be met:
## http://www.opensource.org/licenses/gpl-license.php
##
## If you are unsure which license is appropriate for your use (for
## instance, you are interested in developing a commercial derivative
## of VisTrails), please contact us at vistrails@sci.utah.edu.
##
## This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
## WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
##
############################################################################
import core.modules
import core.modules.module_registry
from core.modules.vistrails_module import Module, ModuleError
import numpy
from scipy import sparse

############################################################################

class Matlab(object):
    my_namespace = 'matlab'

############################################################################

class MatlabSession(Matlab, Module):
    my_namespace = Matlab.my_namespace + '|session'

    def compute(self):
        from mlabwrap import mlab
        self.session = mlab
        self.setResult("Session", self)

    @classmethod
    def register(cls, reg, basic):
        reg.add_module(cls, namespace=cls.my_namespace)
        reg.add_output_port(cls, "Session", (MatlabSession, 'Session'))
        
############################################################################

class MatlabFigure(Matlab, Module):
    my_namespace = Matlab.my_namespace + '|figure'
    
    def is_cacheable(self):
        return False

    def compute(self):
        m = self.getInputFromPort("Session").session
        i = self.getInputFromPort("Number")
        if i > 0:
            m.figure(i)
        else:
            m.figure()
        self.setResult("Figure", self)
        self.setResult("Session",self.getInputFromPort("Session"))

    @classmethod
    def register(cls, reg, basic):
        reg.add_module(cls, namespace=cls.my_namespace)
        reg.add_input_port(cls, "Session", (MatlabSession, 'Session'))
        reg.add_input_port(cls, "Number", (basic.Integer, 'Figure Number'))
        reg.add_output_port(cls, "Figure", (MatlabFigure, 'Figure'))
        reg.add_output_port(cls, "Session", (MatlabSession, 'Session'))

############################################################################

class MatlabPlot(Matlab, Module):
    my_namespace = Matlab.my_namespace + '|figure'

    def compute(self):
        data = self.getInputFromPort("Data")
        m = self.getInputFromPort("Session").session
        m.plot(data.get_array())
        self.setResult("Session",self.getInputFromPort("Session"))

    @classmethod
    def register(cls, reg, basic):
        reg.add_module(cls, namespace=cls.my_namespace)
        reg.add_input_port(cls, "Session", (MatlabSession, 'Session'))
        reg.add_input_port(cls, "Figure", (MatlabFigure, 'Figure'))
        reg.add_input_port(cls, "Data", (reg.get_module_by_name('edu.utah.sci.vistrails.numpyscipy', 'Numpy Array', namespace='numpy|array'), 'Data'))
        reg.add_output_port(cls, "Session", (MatlabSession, 'Session'))

############################################################################

class ClearSession(Matlab, Module):
    my_namespace = Matlab.my_namespace + '|session'

    def compute(self):
        s = self.getInputFromPort("Session")
        s.session.clear()
        self.setResult("Session",s)

    @classmethod
    def register(cls, reg, basic):
        reg.add_module(cls, namespace=cls.my_namespace)
        reg.add_input_port(cls, "Session", (MatlabSession, 'Session'))
        reg.add_output_port(cls, "Session", (MatlabSession, 'Session'))

############################################################################

class MatlabResults(Matlab, Module):
    my_namespace = Matlab.my_namespace + '|data'

    def compute(self):
        pass

    @classmethod
    def register(cls, reg, basic):
        reg.add_module(cls, namespace=cls.my_namespace)

############################################################################

class MatlabRunCommand(Matlab, Module):
    my_namespace = Matlab.my_namespace + '|command'

    def compute(self):
        engine = self.getInputFromPort("Session")
        cmd = self.getInputFromPort("Matlab Command")
        argl = self.getInputFromPort("Arguments").args
        tgt = self.getInputFromPort("LValues")
        reg = core.modules.module_registry
        ar = reg.get_module_by_name('edu.utah.sci.vistrails.numpyscipy', 'Numpy Array', namespace='numpy|array')

        num_out = tgt

        arg_list = []

        for it in argl:
            if type(it) == type(''):
                if it[0:4] == 'mlab':
                    v = it.split('.')
                    arg_list.append(getattr(engine.session, v[1]))
                elif it[0:4] == 'port':
                    v = it.split('.')
                    d = self.getInputFromPort(v[1]).get_array()
                    d = d[0] + 0.0
                    arg_list.append(d)
                else:
                    arg_list.append(it)
            else:
                arg_list.append(it)

        if num_out == 1:
            d = getattr(engine.session, cmd)(nout=num_out,*arg_list)
            t = (d,)
        else:
            t = getattr(engine.session, cmd)(nout=num_out,*arg_list)

        res = MatlabResults()
        res.data = t

        self.setResult("Session", engine)
        self.setResult("Results", res)

    @classmethod
    def register(cls, reg, basic):
        ar = reg.get_module_by_name('edu.utah.sci.vistrails.numpyscipy', 'Numpy Array', namespace='numpy|array')
        reg.add_module(cls, namespace=cls.my_namespace)
        reg.add_input_port(cls, "Session", (MatlabSession, 'Session'))
        reg.add_input_port(cls, "Matlab Command", (basic.String, 'Comamnd'))
        reg.add_input_port(cls, "Arguments", (MatlabArguments, 'Command Arguments'))
        reg.add_input_port(cls, "LValues", (basic.Integer, 'Return Values'))
        reg.add_input_port(cls, "data_0", (ar, 'Input Data 0'))
        reg.add_input_port(cls, "data_1", (ar, 'Input Data 1'))
        reg.add_input_port(cls, "data_2", (ar, 'Input Data 2'), True)
        reg.add_input_port(cls, "data_3", (ar, 'Input Data 3'), True)
        reg.add_input_port(cls, "data_4", (ar, 'Input Data 4'), True)
        reg.add_input_port(cls, "data_5", (ar, 'Input Data 5'), True)
        reg.add_input_port(cls, "data_6", (ar, 'Input Data 6'), True)
        reg.add_output_port(cls, "Session", (MatlabSession, 'Session'))
        reg.add_output_port(cls, "Results", (MatlabResults, 'Results'))

############################################################################
        
class MatlabGetData(Matlab, Module):
    my_namespace = Matlab.my_namespace + '|data'

    def compute(self):
        engine = self.getInputFromPort("Session")
        varname = self.getInputFromPort("VarName")
        localvar = getattr(engine.session, varname)
        ar = reg.get_module_by_name('edu.utah.sci.vistrails.numpyscipy', 'Numpy Array', namespace='numpy|array')
        if type(localvar) == type(0):
            self.setResult("Integer", localvar)
        elif type(localvar) == type(0.0):
            self.setResult("Double", localvar)
        else:
            m = ar()
            m.set_array(numpy.array(localvar.copy()))
            self.setResult("Matrix", m)

        self.setResult("Session", engine)

    @classmethod
    def register(cls, reg, basic):
        ar = reg.get_module_by_name('edu.utah.sci.vistrails.numpyscipy', 'Numpy Array', namespace='numpy|array')
        reg.add_module(cls, namespace=cls.my_namespace)
        reg.add_input_port(cls, "Session", (MatlabSession, 'Session'))
        reg.add_input_port(cls, "VarName", (basic.String, 'Variable Name'))
        reg.add_output_port(cls, "Session", (MatlabSession, 'Session'))
        reg.add_output_port(cls, "Matrix", (ar, 'Output Data'))
        reg.add_output_port(cls, "Integer", (basic.Integer, 'Output Integer'))
        reg.add_output_port(cls, "Double", (basic.Float, 'Output Float'))

            
############################################################################
        
class MatlabSetData(Matlab, Module):
    my_namespace = Matlab.my_namespace + '|data'

    def compute(self):
        engine = self.getInputFromPort("Session")
        varname = self.getInputFromPort("VarName")
        varval = self.getInputFromPort("Value")
        setattr(engine.session, varname, varval)

        self.setResult("Session", engine)
            
    @classmethod
    def register(cls, reg, basic):
        reg.add_module(cls, namespace=cls.my_namespace)
        reg.add_input_port(cls, "Session", (MatlabSession, 'Session'))
        reg.add_input_port(cls, "VarName", (basic.String, 'Matlab Variable Name'))
        reg.add_input_port(cls, "Value", (Module, 'Value'))
        reg.add_output_port(cls, "Session", (MatlabSession, 'Session'))

############################################################################

class MatlabAddPath(Matlab, Module):
    my_namespace = Matlab.my_namespace + '|session'

    def compute(self):
        engine = self.getInputFromPort("Session")
        dirname = self.getInputFromPort("Directory")
        p = mlab.path()
        mlab.path(p, dirname)
        self.setResult("Session", engine)

    @classmethod
    def register(cls, reg, basic):
        reg.add_module(cls, namespace=cls.my_namespace)
        reg.add_input_port(cls, "Session", (MatlabSession, 'Session'))
        reg.add_input_port(cls, "Directory", (basic.String, 'Directory'))

############################################################################

class MatlabArguments(Matlab, Module):
    my_namespace = Matlab.my_namespace + '|command'

    def compute(self):
        arg_list = self.forceGetInputListFromPort("Argument")
        self.args = []
        for v,t in arg_list:
            t = t.upper()
            if t == "FLOAT":
                v = float(v)
            elif t == "INT" or t == "INTEGER":
                v = int(v)
            elif t == "MLAB":
                v = "mlab."+v
            elif t == "PORT":
                v = "port."+v
            else:
                v = str(v)

            self.args.append(v)

        self.setResult("Argument List", self)

    @classmethod
    def register(cls, reg, basic):
        reg.add_module(cls, namespace=cls.my_namespace)
        reg.add_input_port(cls, "Argument", [basic.String, basic.String])
        reg.add_output_port(cls, "Argument List", (MatlabArguments, 'Argument List'))

############################################################################

class MatlabSplitResults(Matlab, Module):
    my_namespace = Matlab.my_namespace + '|data'

    def compute(self):
        import core.modules.module_registry
        from scipy import sparse
        reg = core.modules.module_registry
        ar = reg.get_module_by_name('edu.utah.sci.vistrails.numpyscipy', 'Numpy Array', namespace='numpy|array')

        res = self.getInputFromPort("ResultSet").data
        ind = self.getInputFromPort("DataIndex")

        m = ar()
        m.set_array(numpy.array(res[ind]))

        self.setResult("Matrix", m)

    @classmethod
    def register(cls, reg, basic):
        ar = reg.get_module_by_name('edu.utah.sci.vistrails.numpyscipy', 'Numpy Array', namespace='numpy|array')
        reg.add_module(cls, namespace=cls.my_namespace)
        reg.add_input_port(cls, "ResultSet", (MatlabResults, 'Results'))
        reg.add_input_port(cls, "DataIndex", (basic.Integer, 'Data Index'))
        reg.add_output_port(cls, "Matrix", (ar, 'Array'))
